package com.gic.evaluate.service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.evaluate.dto.EvaluateDTO;
import com.gic.evaluate.dto.MemberReplyDTO;
import com.gic.evaluate.qo.EvaluateOutQO;

import java.util.List;
import java.util.Map;

/**
 * 评价管理好办接口
 * @Description:
 * @author zhiwj
 * @date 2019-12-27 9:20
 */
public interface EvaluateOutApiService {

    /**
     * 同步评价
     * @Title: saveEvaluate
     * @Description:
     * @author zhiwj
     * @param evaluateDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> saveEvaluate(EvaluateDTO evaluateDTO);

    /**
     * 会员追评
     * @Title: saveReview
     * @Description:
     * @author zhiwj
     * @param memberReplyDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> saveReply(MemberReplyDTO memberReplyDTO);

    /**
     * 查询门店下的所有评价（仅包含展示的）
     * @Title: listEvaluateByStore
     * @Description:
     * @author zhiwj
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.evaluate.dto.EvaluateDTO>>
     * @throws
     */
//    ServiceResponse<Page<EvaluateDTO>> listEvaluateByStore(EvaluateOutQO evaluateOutQO);

    /**
     * 查询会员提的所有评价（包含 展示和未展示）
     * @Title: listEvaluateByMember
     * @Description:
     * @author zhiwj
     * @param evaluateOutQO
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.evaluate.dto.EvaluateDTO>>
     * @throws
     */
//    ServiceResponse<List<EvaluateDTO>> listEvaluateByMember(EvaluateOutQO evaluateOutQO);

    /**
     * 查询订单的所有评价
     * @Title: listEvaluateByOrderId
     * @Description:
     * @author zhiwj
     * @param evaluateOutQO
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.evaluate.dto.EvaluateDTO>>
     * @throws
     */
    ServiceResponse<EvaluateDTO> listEvaluateByOrderId(EvaluateOutQO evaluateOutQO);

    /**
     * 查询订单号对应的回复状态 key是订单号， value是状态（-1:未评价  0管理员没有回复过  1管理员回复了   2有新的追评管理员没回复  3逾期不可评价    4不支持追评    6超过最大追评次数 ）
     * @Title: queryRelayStatusByOrderId
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param orderIdList
     * @return com.gic.api.base.commons.ServiceResponse<java.util.Map<java.lang.Long,java.lang.Integer>>
     * @throws
     */
    ServiceResponse<Map<Long, Integer>> queryRelayStatusByOrderId(Integer enterpriseId, List<Long> orderIdList, Long memberId);

    /**
     * 用户隐藏追评
     * @Title: updateShowStatus
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param evaluateReplyId
     * @param memberId
     * @param showStatus
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> updateShowStatus(Integer enterpriseId, Integer evaluateReplyId, Long memberId, Integer showStatus);
}
